@@ -44,11 +44,6 @@ class CollectionFilterProxyModel::Private
 
     bool collectionAccepted( const QModelIndex &index, bool checkResourceVisibility = true );
 
-    void slotReset()
-    {
-      mParent->reset();
-    }
-
     QList< QModelIndex > acceptedResources;
     CollectionFilterProxyModel *mParent;
     MimeTypeChecker mimeChecker;
@@ -69,13 +64,13 @@ bool CollectionFilterProxyModel::Private::collectionAccepted( const QModelIndex
       while ( resource.parent().isValid() )
         resource = resource.parent();
 
-      // See if that resource is visible, if not, reset the model.
+      // See if that resource is visible, if not, invalidate the filter.
       if ( resource != index && !acceptedResources.contains( resource ) ) {
         kDebug() << "We got a new collection:" << mParent->sourceModel()->data( index ).toString()
                  << "but the resource is not visible:" << mParent->sourceModel()->data( resource ).toString();
         acceptedResources.clear();
         // defer reset, the model might still be supplying new items at this point which crashs
-        QTimer::singleShot( 0, mParent, SLOT(slotReset()) );
+        mParent->invalidateFilter();
         return true;
       }
     }
@@ -110,7 +105,6 @@ CollectionFilterProxyModel::CollectionFilterProxyModel( QObject *parent )
   : QSortFilterProxyModel( parent ),
     d( new Private( this ) )
 {
-  setSupportedDragActions( Qt::CopyAction | Qt::MoveAction );
 }
 
 CollectionFilterProxyModel::~CollectionFilterProxyModel()
@@ -147,4 +141,20 @@ void CollectionFilterProxyModel::clearFilters()
   invalidateFilter();
 }
 
+Qt::ItemFlags CollectionFilterProxyModel::flags( const QModelIndex& index ) const
+{
+  if ( !index.isValid() ) {
+    // Don't crash
+    return 0;
+  }
+  
+  const Collection collection = sourceModel()->data( mapToSource( index ), CollectionModel::CollectionRole ).value<Collection>();
+
+  // If this collection directly contains one valid mimetype, it is accepted
+  if ( d->mimeChecker.isWantedCollection( collection ) )
+    return QSortFilterProxyModel::flags( index );
+  else
+    return QSortFilterProxyModel::flags( index ) & ~( Qt::ItemIsSelectable );
+}
+
 #include "collectionfilterproxymodel.moc"
