@@ -145,7 +145,7 @@ ssize_t POP3Protocol::myRead(void *data, ssize_t len)
     memcpy(data, readBuffer, copyLen);
     readBufferLen -= copyLen;
     if (readBufferLen)
-      memcpy(readBuffer, &readBuffer[copyLen], readBufferLen);
+      memmove(readBuffer, &readBuffer[copyLen], readBufferLen);
     return copyLen;
   }
   waitForResponse(600);
@@ -164,7 +164,7 @@ ssize_t POP3Protocol::myReadLine(char *data, ssize_t len)
       data[copyLen] = '\0';
       readBufferLen -= copyLen;
       if (readBufferLen)
-        memcpy(readBuffer, &readBuffer[copyLen], readBufferLen);
+        memmove(readBuffer, &readBuffer[copyLen], readBufferLen);
       return copyLen;
     }
     waitForResponse(600);
@@ -191,7 +191,7 @@ POP3Protocol::Resp POP3Protocol::getResponse(char *r_buf, unsigned int r_len)
   // Clear out the buffer
   memset(buf, 0, r_len);
   myReadLine(buf, r_len - 1);
-  kDebug(7105) << "S:" << buf;
+  //kDebug(7105) << "S:" << buf;
 
   // This is really a funky crash waiting to happen if something isn't
   // null terminated.
@@ -281,7 +281,7 @@ bool POP3Protocol::sendCommand(const QByteArray &cmd)
   QByteArray debugCommand = cmd;
   if (!m_sPass.isEmpty())
     debugCommand.replace(m_sPass.toAscii(),"<password>");
-  kDebug(7105) << "C:" << debugCommand;
+  //kDebug(7105) << "C:" << debugCommand;
 
   // Now actually write the command to the socket
   if (write(cmdrn.data(), cmdrn.size()) != static_cast < ssize_t >
@@ -470,7 +470,7 @@ int POP3Protocol::loginSASL( KIO::AuthInfo &ai )
         myReadLine(buf, sizeof(buf) - 1);
 
         // HACK: This assumes fread stops at the first \n and not \r
-        if (strcmp(buf, ".\r\n") == 0) {
+        if ( (buf[0] == 0) || (strcmp(buf, ".\r\n") == 0) ) {
           break;              // End of data
         }
         // sanders, changed -2 to -1 below
@@ -559,8 +559,8 @@ int POP3Protocol::loginSASL( KIO::AuthInfo &ai )
   if (metaData("auth") == "SASL") {
     closeConnection();
     error(ERR_COULD_NOT_LOGIN,
-          i18n("Your POP3 server does not support SASL.\n"
-               "Choose a different authentication method."));
+          i18n("Your POP3 server (%1) does not support SASL.\n"
+               "Choose a different authentication method.", m_sServer));
     return -1;
   }
   return 1;
@@ -672,8 +672,8 @@ bool POP3Protocol::pop3_open()
 
     if (metaData("auth") == "APOP" && !supports_apop) {
       error(ERR_COULD_NOT_LOGIN,
-          i18n("Your POP3 server does not support APOP.\n"
-               "Choose a different authentication method."));
+          i18n("Your POP3 server (%1) does not support APOP.\n"
+               "Choose a different authentication method.", m_sServer));
       closeConnection();
       return false;
     }
@@ -689,19 +689,18 @@ bool POP3Protocol::pop3_open()
         kDebug(7105) << "TLS mode has been enabled.";
       } else {
         kDebug(7105) << "TLS mode setup has failed. Aborting." << endl;
-        error(ERR_COULD_NOT_CONNECT,
+        error(ERR_SLAVE_DEFINED,
               i18n("Your POP3 server claims to "
                    "support TLS but negotiation "
-                   "was unsuccessful. You can "
-                   "disable TLS in KDE using the "
-                   "crypto settings module."));
+                   "was unsuccessful.\nYou can "
+                   "disable TLS in the POP account settings dialog."));
         closeConnection();
         return false;
       }
     } else if (metaData("tls") == "on") {
-      error(ERR_COULD_NOT_CONNECT,
-            i18n("Your POP3 server does not support TLS. Disable "
-                 "TLS, if you want to connect without encryption."));
+      error(ERR_SLAVE_DEFINED,
+            i18n("Your POP3 server (%1) does not support TLS. Disable "
+                 "TLS, if you want to connect without encryption.", m_sServer));
       closeConnection();
       return false;
     }
@@ -831,7 +830,7 @@ void POP3Protocol::get(const KUrl & url)
         myReadLine(buf, sizeof(buf) - 1);
 
         // HACK: This assumes fread stops at the first \n and not \r
-        if (strcmp(buf, ".\r\n") == 0) {
+        if ( (buf[0] == 0) || (strcmp(buf, ".\r\n") == 0) ) {
           break;                // End of data
         }
         // sanders, changed -2 to -1 below
@@ -897,6 +896,7 @@ void POP3Protocol::get(const KUrl & url)
 
     int activeCommands = 0;
     QStringList::ConstIterator it = waitingCommands.begin();
+    bool firstCommand = true;
     while (it != waitingCommands.end() || activeCommands > 0) {
       while (activeCommands < maxCommands && it != waitingCommands.end()) {
         sendCommand(((cmd ==
@@ -907,7 +907,10 @@ void POP3Protocol::get(const KUrl & url)
       }
       if ( getResponse(buf, sizeof(buf) - 1) == Ok ) {
         activeCommands--;
-        mimeType("message/rfc822");
+        if ( firstCommand ) {
+          firstCommand = false;
+          mimeType("message/rfc822");
+        }
         totalSize(msg_len);
         memset(buf, 0, sizeof(buf));
         char ending = '\n';
